# Detect the distribution in use
%global __despace head -n 1 | tr -d '[:space:]' | sed -e 's/[(].*[)]//g'
%global __lower4 cut -c 1-4 | tr '[:upper:]' '[:lower:]'
%global __distfile %([ -f /etc/SuSE-release ] && echo /etc/SuSE-release || echo /etc/redhat-release)
%global __distinit %(sed -e 's/ release .*//' -e 's/\\([A-Za-z]\\)[^ ]*/\\1/g' %{__distfile} | %{__despace} | %{__lower4})
%global __distvers %(sed -e 's/.* release \\([^. ]*\\).*/\\1/' %{__distfile} | %{__despace})
# Identify CentOS Linux and Scientific Linux as rhel
%if "%{__distinit}" == "c" || "%{__distinit}" == "cl" || "%{__distinit}" == "sl" || "%{__distinit}" == "sls"
%global __distinit rhel
%endif
# Dist tag for Fedora is still "fc"
%if "%{__distinit}" == "f"
%global __distinit fc
%endif

# RPM and CPAN versioning work differently
%global cpanversion 0.2501
%global rpmversion 0.25.01

# We need to tweak the test suite if we have an old version of Test::More
%global old_test_more %(perl -MTest::More -e 'print (($Test::More::VERSION < 0.88) ? 1 : 0);' 2>/dev/null || echo 0)

# Pick up the right dictionary for the spell check
%if 0%{?fedora} > 8 || 0%{?rhel} > 5
%global speller hunspell
%else
%global speller aspell
%endif

Name:		perl-Exception-Base
Epoch:		1
Version:	%{rpmversion}
Release:	2.%{__distinit}%{__distvers}
Summary:	Lightweight exceptions
License:	GPL+ or Artistic
Group:		Development/Libraries
URL:		http://search.cpan.org/dist/Exception-Base/
Source0:	http://search.cpan.org/CPAN/authors/id/D/DE/DEXTER/Exception-Base-%{cpanversion}.tar.gz
Source1:	C4395C9C.pub
Source2:	to_string_changes_errors.t
Patch0:		Exception-Base-0.25-old-Test::more.patch
BuildRoot:	%{_tmppath}/%{name}-%{version}-%{release}-root-%(id -nu)
BuildArch:	noarch
# Module Build
BuildRequires:	coreutils
BuildRequires:	findutils
BuildRequires:	gnupg
BuildRequires:	make
BuildRequires:	perl
%if 0%{?fedora} > 20 || 0%{?rhel} > 7
BuildRequires:	perl-generators
%endif
BuildRequires:	perl(ExtUtils::MakeMaker)
# Module Runtime
BuildRequires:	perl(constant)
BuildRequires:	perl(overload)
BuildRequires:	perl(Scalar::Util)
BuildRequires:	perl(strict)
BuildRequires:	perl(Symbol)
BuildRequires:	perl(warnings)
# Test Suite
BuildRequires:	perl(base)
BuildRequires:	perl(Carp)
BuildRequires:	perl(Cwd)
BuildRequires:	perl(File::Basename)
BuildRequires:	perl(File::Spec)
BuildRequires:	perl(if)
BuildRequires:	perl(Test::Unit::Lite)
# Extra Tests
BuildRequires:	perl(Benchmark)
BuildRequires:	perl(ExtUtils::Manifest)
BuildRequires:	perl(File::Find)
BuildRequires:	perl(File::Slurp)
BuildRequires:	perl(lib)
BuildRequires:	perl(Readonly)
BuildRequires:	perl(Test::Builder)
BuildRequires:	perl(Test::CheckChanges)
BuildRequires:	perl(Test::CPAN::Changes)
BuildRequires:	perl(Test::CPAN::Meta)
BuildRequires:	perl(Test::DistManifest)
BuildRequires:	perl(Test::Distribution)
BuildRequires:	perl(Test::EOL)
BuildRequires:	perl(Test::Kwalitee)
BuildRequires:	perl(Test::MinimumVersion)
BuildRequires:	perl(Test::More)
BuildRequires:	perl(Test::NoTabs)
BuildRequires:	perl(Test::Perl::Critic)
BuildRequires:	perl(Test::Pod) >= 1.14
BuildRequires:	perl(Test::Pod::Coverage) >= 1.04
BuildRequires:	perl(Test::Signature)
BuildRequires:	perl(Test::Spelling), %{speller}-en
BuildRequires:	sed
# Test for https://bugzilla.redhat.com/show_bug.cgi?id=1273668
BuildRequires:	perl(Storable)
# Runtime
Requires:	perl(:MODULE_COMPAT_%(eval "`perl -V:version`"; echo $version))
Requires:	perl(Scalar::Util)
Requires:	perl(Symbol)

%description
This class implements a fully OO exception mechanism similar to
Exception::Class or Class::Throwable. It provides a simple interface
allowing programmers to declare exception classes. These classes can be
thrown and caught. Each uncaught exception prints full stack trace if the
default verbosity is increased for debugging purposes.

%prep
%setup -q -c -n Exception-Base

# Copy up documentation for convenience with %%doc
cp -a Exception-Base-%{cpanversion}/{Changes,examples,Incompatibilities,LICENSE,README} .

# Drop exec bits from examples to avoid doc-file dependencies
chmod -c -x examples/*

# Create a GPG directory for testing, to avoid using ~/.gnupg
mkdir --mode=0700 gnupghome
GNUPGHOME=$(pwd)/gnupghome gpg --import %{SOURCE1}

%build
cd Exception-Base-%{cpanversion}
perl Makefile.PL INSTALLDIRS=vendor
make %{?_smp_mflags}
cd -

%install
rm -rf %{buildroot}
make -C Exception-Base-%{cpanversion} pure_install DESTDIR=%{buildroot}
find %{buildroot} -type f -name .packlist -delete
%{_fixperms} %{buildroot}

%check
export GNUPGHOME=$(pwd)/gnupghome
cd Exception-Base-%{cpanversion}

# === MAIN TEST SUITE === #
make test

# ===  RELEASE TESTS  === #
RELEASE_TESTS="$(echo xt/*.t | sed 's|xt/copyright.t||')"
# Don't run the copyright test as it will fail after the year of module release
RELEASE_TESTS="$(echo $RELEASE_TESTS | sed 's|xt/copyright.t||')"
# Don't run the spelling test as we need to add extra stopwords
RELEASE_TESTS="$(echo $RELEASE_TESTS | sed 's|xt/pod_spell.t||')"
# Some tests need tweaking if we don't have Test::More ≥ 0.88,
# but don't do it yet as it would cause the signature test to fail
%if %{old_test_more}
RELEASE_TESTS="$(echo $RELEASE_TESTS | sed 's|xt/check_changes.t||;s|xt/cpan_changes.t||;s|xt/eol.t||;s|xt/no_tabs.t||')"
%endif

# Signature test would fail on recent distros due to presence of MYMETA.*
[ -f MYMETA.yml ] && mv MYMETA.yml ..
[ -f MYMETA.json ] && mv MYMETA.json ..

make test TEST_FILES="$RELEASE_TESTS"

# Put any MYMETA.* files back where they were
[ -f ../MYMETA.yml ] && mv ../MYMETA.yml .
[ -f ../MYMETA.json ] && mv ../MYMETA.json .

# Now try the Changes test if we didn't run it before
%if %{old_test_more}
patch -p0 < %{P:0}
%if %(perl -e 'print $] >= 5.010000 ? 1 : 0;')
make test TEST_FILES="xt/{check_changes.t,cpan_changes.t,eol.t,no_tabs.t}"
%else
make test TEST_FILES="xt/{check_changes.t,eol.t,no_tabs.t}"
%endif
patch -p0 -R < %{P:0}
%endif
# Fix the POD Spell test and run it
(
	echo "'Unknown"
	echo "'is'"
	echo "'message'"
	echo "'s"
	echo "'value'"
	echo "10x"
	echo "15x"
	echo "exception'"
	echo "informations"
	echo "doesn"
	echo "shouldn"
) >> xt/pod_spellrc
make test TEST_FILES=xt/pod_spell.t

# to_string() appends 'undef' to array attribute
# https://bugzilla.redhat.com/show_bug.cgi?id=1273668
# https://github.com/dex4er/perl-Exception-Base/issues/3
make test TEST_FILES=%{SOURCE2}

cd -

%clean
rm -rf %{buildroot}

%files
%if 0%{?_licensedir:1}
%license LICENSE
%else
%doc LICENSE
%endif
%doc Changes Incompatibilities README examples/
%{perl_vendorlib}/Exception/
%{_mandir}/man3/Exception::Base.3*

%changelog
* Mon Aug 22 2016 Paul Howarth <paul@city-fan.org> - 1:0.25.01-2
- Fix FTBFS when perl is not in the minimal buildroot
- BR: perl-generators where available
- Simplify find command using -delete
- Add test case to verify fix for #1273668

* Sun Nov 22 2015 Paul Howarth <paul@city-fan.org> - 1:0.25.01-1
- Update to 0.2501
  - Fixes "Redundant argument in sprintf" warning on Perl 5.21.2
- Separate RPM and CPAN versioning
- Update upstream GPG key

* Tue Jun 30 2015 Paul Howarth <paul@city-fan.org> - 1:0.25-4
- Fixed test suite failure for Perl 5.21.2 and higher
- Classify buildreqs by usage

* Wed Sep 17 2014 Paul Howarth <paul@city-fan.org> - 1:0.25-3
- BR: perl(Test::CPAN::Changes) unconditionally
- Drop %%defattr, redundant since rpm 4.4
- Use %%license where possible

* Thu Jul 25 2013 Paul Howarth <paul@city-fan.org> - 1:0.25-2
- Perl 5.18 rebuild

* Sun May 26 2013 Paul Howarth <paul@city-fan.org> - 1:0.25-1
- Update to 0.25
  - Builds on Perl 5.18.0 with smartmatch operator as an experimental feature
- Bump epoch due to upstream reverting to 2-digit version number
- Add buildreqs for new release tests

* Sat Jul 14 2012 Paul Howarth <paul@city-fan.org> - 0.2401-6
- Work around failure of signature test due to MYMETA.* files
- BR: perl(base),perl(constant), perl(Cwd) and perl(File::Spec)
- Don't need to remove empty directories from the buildroot
- Drop support for old distributions prior to FC-3:
  - Perl::Critic is always available
  - Don't need to define %%{perl_vendorlib}
  - Use DESTDIR rather than PERL_INSTALL_ROOT
  - Spelling dictionary is never aspell-en-gb

* Fri Jun 24 2011 Paul Howarth <paul@city-fan.org> - 0.2401-4
- Rebuild for perl 5.14.1 in Rawhide
- Fix dist tag for CentOS 6 and Scientific Linux

* Fri May  6 2011 Paul Howarth <paul@city-fan.org> - 0.2401-3
- Import from Fedora

* Tue Feb 08 2011 Fedora Release Engineering <rel-eng@lists.fedoraproject.org> - 0.2401-2
- Rebuilt for https://fedoraproject.org/wiki/Fedora_15_Mass_Rebuild

* Sun Oct 03 2010 Emmanuel Seyman <emmanuel.seyman@club-internet.fr> - 0.2401-1
- Update to 0.2401

* Sat May 01 2010 Marcela Maslanova <mmaslano@redhat.com> - 0.24-3
- Mass rebuild with perl-5.12.0

* Fri Apr 30 2010 Marcela Maslanova <mmaslano@redhat.com> - 0.24-2
- Mass rebuild with perl-5.12.0

* Sun Apr  4 2010 Emmanuel Seyman <emmanuel.seyman@club-internet.fr> - 0.24-1
- Update to 0.24
- Add perl(Test::Unit::Lite) to BuildRequires

* Mon Dec  7 2009 Stepan Kasal <skasal@redhat.com> - 0.21-4
- Rebuild against perl 5.10.1

* Sat Jul 25 2009 Fedora Release Engineering <rel-eng@lists.fedoraproject.org> - 0.21-3
- Rebuilt for https://fedoraproject.org/wiki/Fedora_12_Mass_Rebuild

* Thu Feb 26 2009 Fedora Release Engineering <rel-eng@lists.fedoraproject.org> - 0.21-2
- Rebuilt for https://fedoraproject.org/wiki/Fedora_11_Mass_Rebuild

* Thu Jan 29 2009 Allisson Azevedo <allisson@gmail.com> - 0.21-1
- Initial RPM release
